"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = __importDefault(require("./base"));
const ethers_1 = require("ethers");
const constants_1 = require("../../constants");
const sdk_1 = require("@uniswap/sdk");
const contract = __importStar(require("../../helpers/contracts"));
const types_1 = require("../../types");
class ExchangePancakeSwap extends base_1.default {
    constructor(sellToken, buyToken) {
        super(sellToken, buyToken);
        this.name = types_1.Exchange.pancakeSwap;
    }
    async estimate(sellAmount, provider) {
        console.log('exchange pancake swap');
        let decimals1 = 18;
        let decimals2 = 18;
        let tokenSell;
        let tokenBuy;
        const pancakeSwapFactory = await contract.get(provider, types_1.ExternalContract.pancakeFactoryV2, constants_1.PANCAKE_FACTORY_ADDRESS);
        try {
            if (this.sellToken !== "B&BNB") {
                const erc20instance = await contract.getErc20Address(provider, this.sellToken);
                decimals1 = await erc20instance.decimals();
                tokenSell = new sdk_1.Token(56, this.sellToken, decimals1);
            }
            else {
                tokenSell = new sdk_1.Token(56, constants_1.WBNB_BSC, 18);
            }
            if (this.buyToken !== "B&BNB") {
                const erc20instance = await contract.getErc20Address(provider, this.buyToken);
                decimals2 = await erc20instance.decimals();
                tokenBuy = new sdk_1.Token(56, this.buyToken, decimals2);
            }
            else {
                tokenBuy = new sdk_1.Token(56, constants_1.WBNB_BSC, 18);
            }
        }
        catch (err) {
            throw new Error(`Impossible to construct token ${err}`);
        }
        let pancakePair;
        let route;
        try {
            // create normal PAIR contract
            const pancakePairAddress = await pancakeSwapFactory.getPair(tokenSell.address, tokenBuy.address);
            const pancakePairContract = await contract.get(provider, types_1.ExternalContract.pancakePair, pancakePairAddress);
            const reserves = await pancakePairContract.getReserves();
            const [reserve0, reserve1] = reserves;
            const tokens = [tokenSell, tokenBuy];
            const [token0, token1] = tokens[0].sortsBefore(tokens[1]) ? tokens : [tokens[1], tokens[0]];
            pancakePair = new sdk_1.Pair(new sdk_1.TokenAmount(token0, reserve0), new sdk_1.TokenAmount(token1, reserve1));
            route = new sdk_1.Route([pancakePair], tokenSell);
        }
        catch (err) {
            // if not pair created
            // try with WBNB step
            const WBNBToken = new sdk_1.Token(56, constants_1.WBNB_BSC, 18);
            // create a multiway route: tokento sell --> token to buy
            try {
                // STEP1 Token sell -> WBNB
                const pancakePairAddressStep1 = await pancakeSwapFactory.getPair(this.sellToken, constants_1.WBNB_BSC);
                const pancakePairStep1Contract = await contract.get(provider, types_1.ExternalContract.pancakePair, pancakePairAddressStep1);
                const reservesStep1 = await pancakePairStep1Contract.getReserves();
                const [reserveStep10, reserveStep11] = reservesStep1;
                const tokensStep1 = [tokenSell, WBNBToken];
                const [tokenStep10, tokenStep11] = tokensStep1[0].sortsBefore(tokensStep1[1]) ? tokensStep1 : [tokensStep1[1], tokensStep1[0]];
                const pancakePairStep1 = new sdk_1.Pair(new sdk_1.TokenAmount(tokenStep10, reserveStep10), new sdk_1.TokenAmount(tokenStep11, reserveStep11));
                // STEP2 WBNB -> token buy            
                const pancakePairAddressStep2 = await pancakeSwapFactory.getPair(constants_1.WBNB_BSC, this.buyToken);
                const pancakePairStep2Contract = await contract.get(provider, types_1.ExternalContract.pancakePair, pancakePairAddressStep2);
                const reservesStep2 = await pancakePairStep2Contract.getReserves();
                const [reserveStep20, reserveStep21] = reservesStep2;
                const tokensStep2 = [WBNBToken, tokenBuy];
                const [tokenStep20, tokenStep21] = tokensStep2[0].sortsBefore(tokensStep2[1]) ? tokensStep2 : [tokensStep2[1], tokensStep2[0]];
                const pancakePairStep2 = new sdk_1.Pair(new sdk_1.TokenAmount(tokenStep20, reserveStep20), new sdk_1.TokenAmount(tokenStep21, reserveStep21));
                route = new sdk_1.Route([pancakePairStep1, pancakePairStep2], tokenSell);
            }
            catch (err) {
                throw new Error(`Pancake not supported yet ${err}`);
            }
        }
        const sellAmountWei = ethers_1.ethers.utils.parseUnits(sellAmount, decimals1);
        // console.log('sell amount', sellAmountWei)
        const trade = new sdk_1.Trade(route, new sdk_1.TokenAmount(tokenSell, sellAmountWei.toString()), sdk_1.TradeType.EXACT_INPUT);
        const estimatePrice = Number(trade.executionPrice.toSignificant(6)) * Number(sellAmount);
        // return estimate price =
        return estimatePrice.toString();
    }
    async trade(sellAmount, buyAmount, wallet, txOptions) {
        let decimals1 = 18;
        let decimals2 = 18;
        let tokenSell;
        let tokenBuy;
        const pancakeSwapFactory = await contract.get(wallet.provider, types_1.ExternalContract.pancakeFactoryV2, constants_1.PANCAKE_FACTORY_ADDRESS);
        const pancakeSwapRouter = await contract.get(wallet.provider, types_1.ExternalContract.pancakeRouter, constants_1.PANCAKE_SWAP_ROUTER);
        try {
            if (this.sellToken !== "B&BNB") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.sellToken);
                decimals1 = await erc20instance.decimals();
                tokenSell = new sdk_1.Token(56, this.sellToken, decimals1);
            }
            else {
                tokenSell = new sdk_1.Token(56, constants_1.WBNB_BSC, 18);
            }
            if (this.buyToken !== "B&BNB") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.buyToken);
                decimals2 = await erc20instance.decimals();
                tokenBuy = new sdk_1.Token(56, this.buyToken, decimals2);
            }
            else {
                tokenBuy = new sdk_1.Token(56, constants_1.WBNB_BSC, 18);
            }
        }
        catch (err) {
            throw new Error(`Impossible to construct token ${err}`);
        }
        let pair;
        let route;
        let path;
        let txSend;
        const deadline = Math.floor(Date.now() / 1000) + 60 * 2; // 20 minutes from the current Unix time
        try {
            // if WBNB involved trade, get pair and create route
            if (this.sellToken === "B&BNB") {
                txOptions.value = ethers_1.ethers.utils.parseEther(sellAmount);
                const path = [constants_1.WBNB_BSC, this.buyToken];
                txSend = await pancakeSwapRouter
                    .connect(wallet)
                    .swapExactETHForTokens(ethers_1.ethers.utils.parseUnits(String(buyAmount), decimals2).div(100).mul(95), path, wallet.address, deadline, txOptions);
            }
            else if (this.buyToken === "B&BNB") {
                path = [this.sellToken, constants_1.WBNB_BSC];
                txSend = await pancakeSwapRouter
                    .connect(wallet)
                    .swapExactTokensForETH(ethers_1.ethers.utils.parseUnits(String(sellAmount), decimals1), ethers_1.ethers.utils.parseUnits(String(buyAmount), decimals2).div(100).mul(95), path, wallet.address, deadline, txOptions);
            }
            else {
                // by default choose token1, WBNB, token2
                path = [this.sellToken, constants_1.WBNB_BSC, this.buyToken];
                txSend = await pancakeSwapRouter
                    .connect(wallet)
                    .swapExactTokensForTokens(ethers_1.ethers.utils.parseUnits(sellAmount, decimals1), ethers_1.ethers.utils.parseUnits(buyAmount, decimals2).div(100).mul(50), path, wallet.address, deadline, txOptions);
            }
            // by default choose token1 -> WNBN -> token2
        }
        catch (e) {
            console.log('err pancakeSwap trade', e);
        }
        return txSend;
    }
}
exports.default = ExchangePancakeSwap;
//# sourceMappingURL=pancakeSwap.js.map