"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = __importDefault(require("./base"));
const ethers_1 = require("ethers");
const sdk_1 = require("@uniswap/sdk");
const contract = __importStar(require("../../helpers/contracts"));
const types_1 = require("../../types");
class ExchangeUniswap extends base_1.default {
    constructor(sellToken, buyToken) {
        super(sellToken, buyToken);
        this.name = types_1.Exchange.uniswap;
    }
    async estimate(sellAmount, provider) {
        let decimals1 = 18;
        let decimals2 = 18;
        let tokenSell;
        let tokenBuy;
        try {
            if (this.sellToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(provider, this.sellToken);
                decimals1 = await erc20instance.decimals();
                tokenSell = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.sellToken, decimals1);
            }
            else {
                tokenSell = sdk_1.WETH[1];
            }
            if (this.buyToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(provider, this.buyToken);
                decimals2 = await erc20instance.decimals();
                tokenBuy = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.buyToken, decimals2);
            }
            else {
                tokenBuy = sdk_1.WETH[1];
            }
        }
        catch (err) {
            throw new Error(`Impossible to construct token ${err}`);
        }
        let pair;
        let route;
        try {
            // create normal PAIR contract
            pair = await sdk_1.Fetcher.fetchPairData(tokenSell, tokenBuy);
            route = new sdk_1.Route([pair], tokenSell);
        }
        catch (err) {
            // if not pair created
            // try with ETH step
            try {
                pair = await sdk_1.Fetcher.fetchPairData(tokenSell, sdk_1.WETH[1]);
                const pair2 = await sdk_1.Fetcher.fetchPairData(sdk_1.WETH[1], tokenBuy);
                route = new sdk_1.Route([pair, pair2], tokenSell);
            }
            catch (err) {
                throw new Error(`Uniswap not supported yet ${err}`);
            }
        }
        const sellAmountWei = ethers_1.ethers.utils.parseUnits(sellAmount, decimals1);
        const trade = new sdk_1.Trade(route, new sdk_1.TokenAmount(tokenSell, sellAmountWei.toString()), sdk_1.TradeType.EXACT_INPUT);
        console.log(trade.executionPrice.toSignificant(6));
        console.log(trade.nextMidPrice.toSignificant(6));
        const estimatePrice = Number(trade.executionPrice.toSignificant(6)) * Number(sellAmount);
        // return estimate price =
        return estimatePrice.toString();
    }
    async trade(sellAmount, buyAmount, wallet, txOptions) {
        try {
            let decimals1 = 18;
            let decimals2 = 18;
            let tokenSell;
            let tokenBuy;
            if (this.sellToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.sellToken);
                decimals1 = await erc20instance.decimals();
                tokenSell = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.sellToken, decimals1);
            }
            else {
                tokenSell = sdk_1.WETH[1];
            }
            // token buy
            if (this.buyToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.buyToken);
                decimals2 = await erc20instance.decimals();
                tokenBuy = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.buyToken, decimals2);
            }
            else {
                tokenBuy = sdk_1.WETH[1];
            }
            let pair;
            let route;
            let path;
            try {
                // create normal PAIR contract
                pair = await sdk_1.Fetcher.fetchPairData(tokenSell, tokenBuy);
                route = new sdk_1.Route([pair], tokenSell);
                path = [tokenSell.address, tokenBuy.address];
            }
            catch (err) {
                // if not pair created
                // try with ETH step
                try {
                    pair = await sdk_1.Fetcher.fetchPairData(tokenSell, sdk_1.WETH[1]);
                    const pair2 = await sdk_1.Fetcher.fetchPairData(sdk_1.WETH[1], tokenBuy);
                    route = new sdk_1.Route([pair, pair2], tokenSell);
                    path = [tokenSell.address, sdk_1.WETH[1], tokenBuy.address];
                }
                catch (err) {
                    throw new Error(`Uniswap not supported yet ${err}`);
                }
            }
            // return an estimation
            const sellAmountWei = ethers_1.ethers.utils.parseUnits(sellAmount, decimals1);
            const trade = new sdk_1.Trade(route, new sdk_1.TokenAmount(tokenSell, sellAmountWei.toString()), sdk_1.TradeType.EXACT_INPUT);
            console.log("exec price ", trade.executionPrice.toSignificant(6));
            console.log(trade.nextMidPrice.toSignificant(6));
            const estimatePrice = Number(trade.executionPrice.toSignificant(6)) * Number(sellAmount);
            const uniswapV2Router02Address = await contract.getContractAddress(types_1.ExternalContract.uniswapV2Router02, "homestead");
            const uniswapV2Router02 = await contract.get(wallet.provider, types_1.ExternalContract.uniswapV2Router02, uniswapV2Router02Address);
            // console.log("uniswapV2Router02", uniswapV2Router02);
            let tradeTx;
            const deadline = Math.floor(Date.now() / 1000) + 100; // 100 seconds from now
            const to = wallet.address;
            const slippageTolerance = new sdk_1.Percent("50", "10000"); // 50 bips, or 0.50%
            const amountOutMin = trade.minimumAmountOut(slippageTolerance).raw; // needs to be converted to e.g. hex
            if (this.sellToken === types_1.Token.ETH) {
                txOptions.value = ethers_1.ethers.utils.parseEther(sellAmount);
                txOptions.gasLimit = 250000;
                console.log("txOptions ==> ", txOptions);
                tradeTx = await uniswapV2Router02
                    .connect(wallet)
                    .swapExactETHForTokens(String(amountOutMin), path, to, deadline, txOptions);
            }
            else if (this.buyToken === types_1.Token.ETH) {
                console.log("check 2");
                console.log("path => ", path);
                txOptions.gasLimit = 250000;
                console.log("txOptions ==> ", txOptions);
                tradeTx = await uniswapV2Router02
                    .connect(wallet)
                    .swapExactTokensForETH(sellAmountWei.toString(), String(amountOutMin), path, to, deadline, txOptions);
            }
            else {
                console.log("check 3");
                const slippageTolerance = new sdk_1.Percent("50", "10000"); // 50 bips, or 0.50%
                const amountOutMin = trade.minimumAmountOut(slippageTolerance).raw; // needs to be converted to e.g. hex
                txOptions.gasLimit = 350000;
                // token to token trade
                // find the path
                tradeTx = await uniswapV2Router02.connect(wallet).swapExactTokensForTokens(sellAmountWei.toString(), String(amountOutMin), path, to, deadline, txOptions);
            }
            console.log("tradeTx => ", tradeTx);
            return tradeTx;
        }
        catch (err) {
            throw new Error(`Swap impossible ${err}`);
        }
    }
    async trade_delayed(sellAmount, buyAmount, wallet, nonce, txOptions) {
        try {
            let decimals1 = 18;
            let decimals2 = 18;
            let tokenSell;
            let tokenBuy;
            console.log("trade delayed");
            if (this.sellToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.sellToken);
                decimals1 = await erc20instance.decimals();
                tokenSell = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.sellToken, decimals1);
            }
            else {
                tokenSell = sdk_1.WETH[1];
            }
            // token buy
            if (this.buyToken !== "ETH") {
                const erc20instance = await contract.getErc20Address(wallet.provider, this.buyToken);
                decimals2 = await erc20instance.decimals();
                tokenBuy = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.buyToken, decimals2);
            }
            else {
                tokenBuy = sdk_1.WETH[1];
            }
            let pair;
            let route;
            let path;
            try {
                // create normal PAIR contract
                pair = await sdk_1.Fetcher.fetchPairData(tokenSell, tokenBuy);
                route = new sdk_1.Route([pair], tokenSell);
                path = [tokenSell.address, tokenBuy.address];
            }
            catch (err) {
                // if not pair created
                // try with ETH step
                try {
                    pair = await sdk_1.Fetcher.fetchPairData(tokenSell, sdk_1.WETH[1]);
                    const pair2 = await sdk_1.Fetcher.fetchPairData(sdk_1.WETH[1], tokenBuy);
                    route = new sdk_1.Route([pair, pair2], tokenSell);
                    path = [tokenSell.address, sdk_1.WETH[1], tokenBuy.address];
                }
                catch (err) {
                    throw new Error(`Uniswap not supported yet ${err}`);
                }
            }
            // return an estimation
            const sellAmountWei = ethers_1.ethers.utils.parseUnits(sellAmount, decimals1);
            const trade = new sdk_1.Trade(route, new sdk_1.TokenAmount(tokenSell, sellAmountWei.toString()), sdk_1.TradeType.EXACT_INPUT);
            const estimatePrice = Number(trade.executionPrice.toSignificant(6)) * Number(sellAmount);
            const uniswapV2Router02Address = await contract.getContractAddress(types_1.ExternalContract.uniswapV2Router02, "homestead");
            const uniswapV2Router02 = await contract.get(wallet.provider, types_1.ExternalContract.uniswapV2Router02, uniswapV2Router02Address);
            // console.log("uniswapV2Router02", uniswapV2Router02);
            let tradeTx;
            const deadline = Math.floor(Date.now() / 1000) + 100; // 100 seconds from now
            const to = wallet.address;
            const slippageTolerance = new sdk_1.Percent("50", "10000"); // 50 bips, or 0.50%
            const amountOutMin = trade.minimumAmountOut(slippageTolerance).raw; // needs to be converted to e.g. hex
            const IuniswapV2Router02ABI = await contract.getAbi(types_1.ExternalContract.uniswapV2Router02);
            const IuniswapV2Router02 = new ethers_1.ethers.utils.Interface(IuniswapV2Router02ABI);
            if (this.sellToken === types_1.Token.ETH) {
                const args = Object.values({
                    amountOutMin: String(amountOutMin),
                    path,
                    to,
                    deadline,
                });
                const data = IuniswapV2Router02.functions.swapExactETHForTokens.encode(args);
                const tsx = {
                    nonce,
                    gasPrice: txOptions.gasPrice ? txOptions.gasPrice : 10000000000,
                    gasLimit: 300000,
                    to: uniswapV2Router02Address,
                    value: ethers_1.ethers.utils.parseEther(sellAmount),
                    data,
                    chainId: sdk_1.ChainId.MAINNET,
                };
                return wallet.sign(tsx);
            }
            else if (this.buyToken === types_1.Token.ETH) {
                const args = Object.values({
                    amountIn: sellAmountWei.toString(),
                    amountOutMin: String(amountOutMin),
                    path,
                    to,
                    deadline,
                });
                const data = IuniswapV2Router02.functions.swapExactTokensForETH.encode(args);
                const tsx = {
                    nonce,
                    gasPrice: txOptions.gasPrice ? txOptions.gasPrice : 10000000000,
                    gasLimit: 300000,
                    to: uniswapV2Router02Address,
                    value: 0,
                    data,
                    chainId: sdk_1.ChainId.MAINNET,
                };
                return wallet.sign(tsx);
            }
            else {
                const args = Object.values({
                    amountIn: sellAmountWei.toString(),
                    amountOutMin: String(amountOutMin),
                    path,
                    to,
                    deadline,
                });
                const data = IuniswapV2Router02.functions.swapExactTokensForTokens.encode(args);
                const tsx = {
                    nonce,
                    gasPrice: txOptions.gasPrice ? txOptions.gasPrice : 10000000000,
                    gasLimit: 300000,
                    to: uniswapV2Router02Address,
                    value: 0,
                    data,
                    chainId: sdk_1.ChainId.MAINNET,
                };
                return wallet.sign(tsx);
            }
            // return tradeTx;
        }
        catch (err) {
            throw new Error(`Dealyaed swap impossible ${err}`);
        }
    }
    async tradeFromSell(sellAmount, buyAmount, destAddress, wallet, txOptions) {
        try {
            let decimals1 = 18;
            const tokenSell = sdk_1.WETH[1];
            // token buy
            const erc20instance = await contract.getErc20Address(wallet.provider, this.buyToken);
            decimals1 = await erc20instance.decimals();
            const tokenBuy = new sdk_1.Token(sdk_1.ChainId.MAINNET, this.buyToken, decimals1);
            let pair;
            let route;
            let path;
            try {
                // create normal PAIR contract
                pair = await sdk_1.Fetcher.fetchPairData(tokenSell, tokenBuy);
                route = new sdk_1.Route([pair], tokenSell);
                path = [tokenSell.address, tokenBuy.address];
            }
            catch (err) {
                throw new Error(`Trade not supported yet ${err}`);
            }
            // return an estimation
            const sellAmountWei = ethers_1.ethers.utils.parseUnits(sellAmount, decimals1);
            const trade = new sdk_1.Trade(route, new sdk_1.TokenAmount(tokenSell, sellAmountWei.toString()), sdk_1.TradeType.EXACT_INPUT);
            const estimatePrice = Number(trade.executionPrice.toSignificant(6)) * Number(sellAmount);
            const uniswapV2Router02Address = await contract.getContractAddress(types_1.ExternalContract.uniswapV2Router02, "homestead");
            const uniswapV2Router02 = await contract.get(wallet.provider, types_1.ExternalContract.uniswapV2Router02, uniswapV2Router02Address);
            let tradeTx;
            const deadline = Math.floor(Date.now() / 1000) + 100; // 100 seconds from now
            const to = destAddress;
            const slippageTolerance = new sdk_1.Percent("50", "10000"); // 50 bips, or 0.50%
            const amountOutMin = trade.minimumAmountOut(slippageTolerance).raw; // needs to be converted to e.g. hex
            txOptions.value = ethers_1.ethers.utils.parseEther(sellAmount);
            txOptions.gasLimit = 250000;
            tradeTx = await uniswapV2Router02
                .connect(wallet)
                .swapExactETHForTokens(String(amountOutMin), path, to, deadline, txOptions);
            console.log("sellcrypto tradeTx => ", tradeTx);
            return tradeTx;
        }
        catch (err) {
            throw new Error(`Swap impossible ${err}`);
        }
    }
}
exports.default = ExchangeUniswap;
//# sourceMappingURL=uniswap.js.map