"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteUserShop = exports.collectShopTaxes = exports.setShopLicencePrice = exports.topUpShop = exports.removeShop = exports.addShop = exports.getShopsInZones = exports.getShopsInZone = exports.getShopByPosition = exports.getShopByAddress = exports.getLicencePrice = exports.existsByAddress = exports.createShopTopUpBytes = exports.createShopBytes = exports.shopArrToObj = void 0;
const ethers_1 = require("ethers");
const constants = __importStar(require("../constants"));
const convert = __importStar(require("../helpers/convert"));
const util = __importStar(require("../helpers/util"));
const validate = __importStar(require("../helpers/validate"));
const SHOP_ADD_FN = "30";
// -------------------- //
//       Formatters     //
// -------------------- //
exports.shopArrToObj = (shopArr) => {
    const shopObj = {
        position: "",
        zoneGeohash: "",
        category: "",
        name: "",
        description: "",
        opening: "",
        // hasDispute: false,
        // disputeID: undefined,
        staked: "",
    };
    try {
        shopObj.position = convert.hexToAscii(shopArr[0]);
    }
    catch (e) {
        shopObj.position = "ujxs37uuuuuu";
    }
    try {
        shopObj.zoneGeohash = convert.hexToAscii(`0x${convert.remove0x(shopArr[0]).slice(0, 12)}`);
    }
    catch (e) {
        shopObj.zoneGeohash = "ujxs37";
    }
    try {
        shopObj.category =
            shopArr[1] !== constants.BYTES16_ZERO
                ? convert.hexToAscii(shopArr[1])
                : undefined;
    }
    catch (e) {
        console.log("error shopArrToObj category ", e, shopArr[1]);
        shopObj.category = "--";
    }
    try {
        shopObj.name =
            shopArr[2] !== constants.BYTES16_ZERO
                ? convert.hexToAscii(shopArr[2])
                : "--";
    }
    catch (error) {
        console.log("error shopArrToObj name ", error, shopArr[2]);
        shopObj.name = "--";
    }
    try {
        shopObj.description =
            shopArr[3] !== constants.BYTES32_ZERO && convert.hexToAscii(shopArr[3])
                ? convert.hexToAscii(shopArr[3])
                : "--";
    }
    catch (error) {
        console.log("error shopArrToObj description ", error, shopArr[3]);
        shopObj.description = "--";
    }
    try {
        shopObj.opening =
            shopArr[4] !== constants.BYTES16_ZERO
                ? convert.hexToAscii(shopArr[4])
                : "WWWWWWWWWWWWWW";
    }
    catch (e) {
        shopObj.opening = "--";
    }
    try {
        shopObj.staked = ethers_1.ethers.utils.formatEther(shopArr[5].toString());
        shopObj.lastTaxTime = Number(shopArr[6]);
        shopObj.selfZonePrice = ethers_1.ethers.utils.formatEther(shopArr[7]);
    }
    catch (e) {
        console.log("error get shop ", e);
    }
    return shopObj;
};
// to send as erc233 call data, which calls shop.tokenFallback
exports.createShopBytes = (shopData) => {
    const data = [
        SHOP_ADD_FN,
        util.toNBytes(shopData.country, 2),
        util.toNBytes(shopData.position, 12),
        shopData.category
            ? util.toNBytes(shopData.category, 16)
            : util.remove0x(constants.BYTES16_ZERO),
        shopData.name
            ? util.toNBytes(shopData.name, 16)
            : util.remove0x(constants.BYTES16_ZERO),
        shopData.description
            ? util.toNBytes(shopData.description, 32)
            : util.remove0x(constants.BYTES32_ZERO),
        shopData.opening
            ? util.toNBytes(shopData.opening, 16)
            : util.remove0x(constants.BYTES16_ZERO),
    ].join("");
    return `0x${data}`;
};
// to send as erc233 call data, which calls shop.tokenFallback
exports.createShopTopUpBytes = () => {
    return `0x31${util.toNBytes("0", 94)}`;
};
// -------------------- //
//        Getters       //
// -------------------- //
exports.existsByAddress = async (shopAddress, shopInstance) => {
    validate.ethAddress(shopAddress);
    const shopExists = await shopInstance.shopByAddrExists(shopAddress);
    return shopExists;
};
// untested
exports.getLicencePrice = async (geohash6, shopInstance) => {
    validate.geohash(geohash6, 6);
    try {
        let priceRaw = await shopInstance.zoneLicencePrice(`0x${util.toNBytes(geohash6, 6)}`);
        let price;
        if (Number(ethers_1.ethers.utils.formatEther(priceRaw)) <= 42) {
            price = "42";
        }
        else {
            price = ethers_1.ethers.utils.formatEther(priceRaw);
        }
        return price;
    }
    catch (e) {
        console.log("erreur getLicencePrice()", e);
    }
};
exports.getShopByAddress = async (shopAddress, shopInstance) => {
    validate.ethAddress(shopAddress);
    const shop = exports.shopArrToObj(await shopInstance.getShopByAddr(shopAddress));
    shop.address = shopAddress;
    if (shop.zoneGeohash) {
        const licencePrice = await exports.getLicencePrice(shop.zoneGeohash, shopInstance);
        shop.zonePrice = licencePrice;
    }
    return shop;
};
// Do not return neither address or shopZonePrice
exports.getShopByPosition = async (geohash12, shopInstance) => {
    validate.geohash(geohash12, 12);
    const shop = exports.shopArrToObj(await shopInstance.getShopByPos(convert.asciiToHex(geohash12).substring(0, 26)));
    return shop;
};
exports.getShopsInZone = async (geohash6, shopInstance) => {
    validate.geohash(geohash6, 6);
    const shopAddressesInZone = await shopInstance.getShopAddressesInZone(util.stringToBytes(geohash6.slice(0, 6), 6));
    return Promise.all(shopAddressesInZone.map((shopAddress) => exports.getShopByAddress(shopAddress, shopInstance)));
};
exports.getShopsInZones = async (geohash6List, shopInstance) => Promise.all(geohash6List.map((geohash6) => exports.getShopsInZone(geohash6, shopInstance)));
// -------------------- //
//        Setters       //
// -------------------- //
// ERC223
exports.addShop = async (shopData, shopContract, detherTokenContract, wallet, txOptions) => {
    validate.countryCode(shopData.country);
    validate.geohash(shopData.position, 12);
    validate.zonePrice(shopData.staking);
    // other 4 args are optional strings: category, name, description, opening
    return detherTokenContract
        .connect(wallet)
        .transferAndCall(shopContract.address, ethers_1.ethers.utils.parseEther(shopData.staking), exports.createShopBytes(shopData), txOptions); // erc223 call
};
// currently 1 address can only own 1 shop
exports.removeShop = async (shopContract, wallet, txOptions) => {
    const shopExists = await shopContract.shopByAddrExists(wallet.address);
    if (!shopExists)
        throw new Error("wallet address not registered as shop");
    return shopContract.connect(wallet).removeShop(txOptions);
};
exports.topUpShop = async (topUpAmount, shopContract, detherTokenContract, wallet, txOptions) => {
    return detherTokenContract.connect(wallet).transferAndCall(shopContract.address, ethers_1.ethers.utils.parseEther(topUpAmount), `0x31${util.toNBytes("0", 94)}`, // shop's tokenFallback need 95 bytes as data
    txOptions); // erc223 call
};
exports.setShopLicencePrice = async (geohash6, newPrice, shopContract, wallet, txOptions) => {
    return shopContract
        .connect(wallet)
        .setZoneLicensePrice(`0x${util.toNBytes(geohash6, 6)}`, ethers_1.ethers.utils.parseEther(newPrice), txOptions);
};
exports.collectShopTaxes = async (geohash6, start, end, shopContract, wallet, txOptions) => {
    return shopContract
        .connect(wallet)
        .collectTax(`0x${util.toNBytes(geohash6, 6)}`, start, end, txOptions);
};
exports.deleteUserShop = async (geohash6, shopAddress, shopContract, wallet, txOptions) => {
    return shopContract
        .connect(wallet)
        .removeShopFromZoneOwner(shopAddress, `0x${util.toNBytes(geohash6, 6)}`, txOptions);
};
//# sourceMappingURL=shop.js.map