"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addComment = exports.addFunds = exports.removeTeller = exports.updateTeller = exports.addTeller = exports.isTeller = exports.getTellersInZones = exports.getTellerInZone = exports.getTeller = exports.settingsToBytes = exports.tellerArrToObj = void 0;
const ethers_1 = require("ethers");
const constants = __importStar(require("../constants"));
const util = __importStar(require("../helpers/util"));
const validate = __importStar(require("../helpers/validate"));
const convert = __importStar(require("../helpers/convert"));
const contract = __importStar(require("../helpers/contracts"));
const types_1 = require("../types");
const EMPTY_MESSENGER_FIELD = "0x00000000000000000000000000000000";
// -------------------- //
//      Formatters      //
// -------------------- //
exports.tellerArrToObj = (zoneAddr, tellerContractAddr, tellerArr) => {
    const settingsInt = parseInt(tellerArr[4], 16);
    const isSeller = (settingsInt & 1) > 0;
    const isBuyer = (settingsInt & 2) > 0;
    const description = convert.hexToAscii(tellerArr[8]).split("-");
    return {
        isSeller,
        isBuyer,
        currencyId: tellerArr[1],
        tellerGeohash: convert.hexToAscii(tellerArr[3]),
        zoneAddress: zoneAddr,
        zoneGeohash: convert.hexToAscii(tellerArr[3]).slice(0, 6),
        // optional
        buyRate: isBuyer ? tellerArr[5] / 100 : 0,
        sellRate: isSeller ? tellerArr[6] / 100 : 0,
        referrer: tellerArr[7] !== constants.ADDRESS_ZERO ? tellerArr[7] : undefined,
        messenger: tellerArr[2] !== EMPTY_MESSENGER_FIELD
            ? convert.hexToAscii(tellerArr[2])
            : undefined,
        tellerAddress: tellerArr[0],
        tellerContractAddress: tellerContractAddr,
        sellUp: description.length == 3 ? description[0] : "?",
        buyUp: description.length == 3 ? description[1] : "?",
        ticker: description.length == 3 ? description[2] : "",
    };
};
exports.settingsToBytes = (isBuyer, isSeller) => {
    let settings = "0x0";
    if (isBuyer && isSeller)
        settings += "3";
    else if (isBuyer)
        settings += "2";
    else if (isSeller)
        settings += "1";
    else
        settings += "0";
    return settings;
};
// -------------------- //
//        Getters       //
// -------------------- //
exports.getTeller = async (address, provider, zoneFactoryContract) => {
    validate.ethAddress(address);
    let zoneAddress, tellerAddress, tellerInstance;
    try {
        zoneAddress = await zoneFactoryContract.ownerToZone(address);
        // if (zoneAddr)
        const zoneInstance = await contract.get(provider, types_1.DetherContract.Zone, zoneAddress);
        tellerAddress = await zoneInstance.teller();
        tellerInstance = await contract.get(provider, types_1.DetherContract.Teller, tellerAddress);
    }
    catch (_a) {
        return false;
    }
    const teller = exports.tellerArrToObj(zoneAddress, tellerAddress, await tellerInstance.getTeller());
    if (teller.tellerAddress !== address) {
        return false;
    }
    return teller;
};
exports.getTellerInZone = async (geohash6, provider, zoneFactoryContract) => {
    validate.geohash(geohash6, 6);
    let zoneAddress, tellerAddress, tellerInstance;
    try {
        zoneAddress = await zoneFactoryContract.geohashToZone(convert.asciiToHex(geohash6).substring(0, 14));
        const zoneInstance = await contract.get(provider, types_1.DetherContract.Zone, zoneAddress);
        tellerAddress = await zoneInstance.teller();
        tellerInstance = await contract.get(provider, types_1.DetherContract.Teller, tellerAddress);
    }
    catch (_a) {
        return {};
    }
    return exports.tellerArrToObj(zoneAddress, tellerAddress, await tellerInstance.getTeller());
};
exports.getTellersInZones = async (geohash6List, provider, zoneFactoryContract) => Promise.all(geohash6List.map((geohash6) => exports.getTellerInZone(geohash6, provider, zoneFactoryContract)));
exports.isTeller = async (address, provider, zoneFactoryContract) => {
    validate.ethAddress(address);
    let zoneAddress, tellerAddress, tellerInstance;
    try {
        zoneAddress = await zoneFactoryContract.ownerToZone(address);
        // if (zoneAddr)
        const zoneInstance = await contract.get(provider, types_1.DetherContract.Zone, zoneAddress);
        tellerAddress = await zoneInstance.teller();
        tellerInstance = await contract.get(provider, types_1.DetherContract.Teller, tellerAddress);
    }
    catch (_a) {
        return false;
    }
    const teller = exports.tellerArrToObj(zoneAddress, tellerAddress, await tellerInstance.getTeller());
    if (teller.tellerAddress !== address) {
        return false;
    }
    return teller.tellerAddress;
};
// -------------------- //
//        Setters       //
// -------------------- //
// tellerData contains the zone geohash, the first 6 characters of tellerData.position
exports.addTeller = async (tellerData, wallet, zoneFactoryContract, txOptions) => {
    validate.geohash(tellerData.position, 12);
    validate.currencyId(tellerData.currencyId);
    validate.tellerBuyerInfo(tellerData.isBuyer, tellerData.buyRate);
    validate.tellerSellerInfo(tellerData.isSeller, tellerData.sellRate);
    validate.tellerDescrInfo(tellerData.description);
    if (tellerData.refFees)
        validate.refFees(tellerData.refFees);
    if (tellerData.messenger)
        validate.telegramUsername(tellerData.messenger);
    if (tellerData.referrer)
        validate.ethAddress(tellerData.referrer);
    if (tellerData.description)
        validate.tellerDescrInfo(tellerData.description);
    const tellerSettings = exports.settingsToBytes(tellerData.isBuyer, tellerData.isSeller);
    const geohash6 = tellerData.position.slice(0, 6);
    /*  const zoneFactoryContract = await contract.get(
      wallet.provider,
      DetherContract.ZoneFactory
    );*/
    const zoneAddress = await zoneFactoryContract.geohashToZone(util.stringToBytes(geohash6, 6));
    const zoneContract = await contract.get(wallet.provider, types_1.DetherContract.Zone, zoneAddress);
    const tellerAddress = await zoneContract.teller();
    const tellerContract = await contract.get(wallet.provider, types_1.DetherContract.Teller, tellerAddress);
    return tellerContract
        .connect(wallet)
        .addTeller(util.stringToBytes(tellerData.position, 12), tellerData.currencyId, tellerData.messenger
        ? util.stringToBytes(tellerData.messenger, 16)
        : EMPTY_MESSENGER_FIELD, tellerData.sellRate, tellerData.buyRate, tellerSettings, tellerData.referrer || constants.ADDRESS_ZERO, tellerData.refFees || 0, tellerData.description
        ? util.stringToBytes(tellerData.description, 32)
        : "", txOptions);
};
// tellerData contains the zone geohash, the first 6 characters of tellerData.position
exports.updateTeller = async (tellerData, wallet, zoneFactoryContract, txOptions) => {
    // console.log('teller update teller', tellerData);
    validate.geohash(tellerData.position, 12);
    validate.currencyId(tellerData.currencyId);
    validate.tellerBuyerInfo(tellerData.isBuyer, tellerData.buyRate);
    validate.tellerSellerInfo(tellerData.isSeller, tellerData.sellRate);
    validate.tellerDescrInfo(tellerData.description);
    console.log("dether.js update teller", tellerData);
    if (tellerData.description)
        validate.tellerDescrInfo(tellerData.description);
    const tellerSettings = exports.settingsToBytes(tellerData.isBuyer, tellerData.isSeller);
    const geohash6 = tellerData.position.slice(0, 6);
    const zoneAddress = await zoneFactoryContract.geohashToZone(util.stringToBytes(geohash6, 6));
    const zoneContract = await contract.get(wallet.provider, types_1.DetherContract.Zone, zoneAddress);
    const tellerAddress = await zoneContract.teller();
    const tellerContract = await contract.get(wallet.provider, types_1.DetherContract.Teller, tellerAddress);
    return tellerContract
        .connect(wallet)
        .updateTeller(util.stringToBytes(tellerData.position, 12), tellerData.currencyId, tellerData.messenger
        ? util.stringToBytes(tellerData.messenger, 16)
        : EMPTY_MESSENGER_FIELD, tellerData.sellRate, tellerData.buyRate, tellerSettings, tellerData.description
        ? util.stringToBytes(tellerData.description, 32)
        : "", txOptions);
};
exports.removeTeller = async (zoneGeohash, wallet, zoneFactoryContract, txOptions) => {
    validate.geohash(zoneGeohash, 6);
    const zoneAddress = await zoneFactoryContract.geohashToZone(convert.asciiToHex(zoneGeohash).substring(0, 14));
    const zoneContract = await contract.get(wallet.provider, types_1.DetherContract.Zone, zoneAddress);
    const tellerAddress = await zoneContract.teller();
    const tellerContract = await contract.get(wallet.provider, types_1.DetherContract.Teller, tellerAddress);
    return tellerContract.connect(wallet).removeTeller(txOptions);
};
// ethAmount in wei
exports.addFunds = async (zoneGeohash, ethAmount, wallet, txOptions) => {
    validate.geohash(zoneGeohash, 6);
    const zoneFactoryContract = await contract.get(wallet.provider, types_1.DetherContract.ZoneFactory);
    const zoneAddress = await zoneFactoryContract.geohashToZone(convert.asciiToHex(zoneGeohash).substring(0, 14));
    const zoneContract = await contract.get(wallet.provider, types_1.DetherContract.Zone, zoneAddress);
    const tellerAddress = await zoneContract.teller();
    const tellerContract = await contract.get(wallet.provider, types_1.DetherContract.Teller, tellerAddress);
    const ethAmountBN = ethers_1.ethers.utils.parseUnits(ethAmount, "wei");
    // TODO: check that wallet address has enough eth balance + is a teller
    txOptions.value = ethAmountBN;
    return tellerContract.connect(wallet).addFunds(txOptions);
    // return tellerContract
    //   .connect(wallet)
    //   .addFunds({ ...txOptions, value: ethAmountBN });
};
// --- Comments
exports.addComment = async (zoneGeohash, ipfsHash, wallet, zoneFactoryContract, txOptions) => {
    validate.geohash(zoneGeohash, 6);
    const zoneAddress = await zoneFactoryContract.geohashToZone(convert.asciiToHex(zoneGeohash).substring(0, 14));
    const zoneContract = await contract.get(wallet.provider, types_1.DetherContract.Zone, zoneAddress);
    const tellerAddress = await zoneContract.teller();
    const tellerContract = await contract.get(wallet.provider, types_1.DetherContract.Teller, tellerAddress);
    // TODO: check that wallet address is allowed to place a comment
    return tellerContract
        .connect(wallet)
        .addComment(util.ipfsHashToBytes32(ipfsHash), txOptions);
};
//# sourceMappingURL=teller.js.map