"use strict";
/**
 * @packageDocumentation
 * @module detherJS
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.approveTokenBsc = exports.approveToken = exports.sendCrypto = exports.execTrade_delayed = exports.execTradeFromSell = exports.execTradeBsc = exports.execTrade = exports.getUniswapLiquidity = exports.isExchangeAvailable = exports.hasApprovalBsc = exports.hasApproval = exports.getAvailableTokenDecimals = exports.getAvailableToken = exports.getExchangeEstimationBsc = exports.getExchangeEstimation = exports.getAllBalance = exports.getERC20Info = void 0;
const ethers_1 = require("ethers");
const sdk_1 = require("@uniswap/sdk");
const constants = __importStar(require("../constants"));
const validate = __importStar(require("../helpers/validate"));
const contract = __importStar(require("../helpers/contracts"));
const convert = __importStar(require("../helpers/convert"));
const exchanges = __importStar(require("./exchanges"));
const types_1 = require("../types");
// -------------------- //
//        Getters       //
// -------------------- //
const getBalance = async (address, ticker, provider, bscProvider) => {
    validate.ethAddress(address);
    const result = {};
    const re = /B&/;
    const isBsc = re.test(Object.keys(ticker)[0]);
    if (Object.keys(ticker)[0] === "KSM" || Object.keys(ticker)[0] === "DOT") {
        return;
    }
    if (Object.keys(ticker)[0] === "BTC") {
        return;
    }
    if (Object.keys(ticker)[0] === types_1.Token.ETH && !isBsc) {
        result.ETH = ethers_1.ethers.utils.formatEther(await provider.getBalance(address));
        return result;
    }
    else if (Object.keys(ticker)[0] === 'B&BNB' && isBsc) {
        result['B&BNB'] = ethers_1.ethers.utils.formatEther(await bscProvider.getBalance(address));
        return result;
    }
    let decimals;
    let erc20instance;
    try {
        erc20instance = await contract.getErc20Address(isBsc ? bscProvider : provider, ticker[Object.keys(ticker)[0]]);
    }
    catch (e) {
        console.log("error detherJS getBalance instanciation", e, ticker);
        return;
    }
    try {
        decimals = await erc20instance.decimals();
    }
    catch (e) {
        decimals = 18;
        return;
    }
    try {
        result[`${Object.keys(ticker)[0]}`] = ethers_1.ethers.utils.formatUnits(await erc20instance.balanceOf(address), decimals);
    }
    catch (e) {
        console.log("error detherJS getBalance", e);
        return;
    }
    return result;
};
exports.getERC20Info = async (address, network, provider, bscProvider) => {
    let erc20instance;
    try {
        const tokenInfo = {};
        const erc20instance = network === 'ETH' ? await contract.getErc20Address(provider, address) : await contract.getErc20Address(bscProvider, address);
        const name = await erc20instance.name();
        const symbol = await erc20instance.symbol();
        tokenInfo.address = address;
        tokenInfo.name = name;
        tokenInfo.symbol = symbol;
        return tokenInfo;
    }
    catch (e) {
        throw new Error(`Unable to get info from this address => ${e}`);
    }
};
exports.getAllBalance = async (address, tickers, provider, bscProvider) => {
    const myBalances = await Promise.all(tickers.map((ticker) => getBalance(address, ticker, provider, bscProvider)));
    const cleanBalances = await myBalances.filter(function (el) {
        return el != null;
    });
    return cleanBalances;
};
exports.getExchangeEstimation = async (sellToken, buyToken, sellAmount, provider) => {
    // validate.sellAmount(sellAmount);
    const exchange = exchanges.load(sellToken, buyToken);
    const buyAmountEstimation = await exchange.estimate(sellAmount, provider);
    return buyAmountEstimation.toString();
};
exports.getExchangeEstimationBsc = async (sellToken, buyToken, sellAmount, provider) => {
    // validate.sellAmount(sellAmount);
    const exchange = exchanges.load(sellToken, buyToken, true);
    const buyAmountEstimation = await exchange.estimate(sellAmount, provider);
    return buyAmountEstimation.toString();
};
// TO DO get token address call from staking smart contracts
exports.getAvailableToken = async (provider, forLogo) => {
    // forLogo is for returning mainnet ticker in case of testnet, because we use an open library matching
    // mainnet address to logo
    if (forLogo)
        return constants.TICKER["homestead"];
    else {
        const network = await provider.getNetwork();
        return constants.TICKER[network.name];
    }
};
const _getTokenInfo = async (token, provider) => {
    try {
        // get name
        let erc20instance;
        let newTokenObj = {};
        try {
            erc20instance = await contract.getErc20Address(provider, token.tokenAddress);
            const symbol = await erc20instance.symbol();
            const name = await erc20instance.name();
            newTokenObj.address = token.tokenAddress;
            newTokenObj.logoUrl = token.logoUrl;
            newTokenObj.websiteUrl = token.websiteUrl;
            newTokenObj.community = "on";
            newTokenObj.name = name;
            newTokenObj.symbol = symbol;
            return newTokenObj;
        }
        catch (e) {
            console.log("error detherJS getter token info instanciation", e);
            return;
        }
        return token;
    }
    catch (e) {
        console.log("error getTokenInfo with this url as params", e, token);
    }
};
// TO DO get token address call from staking smart contracts
exports.getAvailableTokenDecimals = async (provider, forLogo) => {
    // forLogo is for returning mainnet ticker in case of testnet, because we use an open library matching
    // mainnet address to logo
    if (forLogo)
        return constants.TICKER["homestead"];
    else {
        // const network = await provider.getNetwork();
        const tokenRegistryInstance = await contract.get(provider, types_1.DetherContract.TokenRegistry);
        const availableToken = await tokenRegistryInstance.getTokenList();
        return Promise.all(availableToken.map((token) => _getTokenInfo(token, provider)));
    }
};
exports.hasApproval = async (owner, sellToken, amount, provider) => {
    const erc20instance = await contract.getErc20Address(provider, sellToken);
    // const exchangeAddress = await contract.getUniswapExchangeAddress(
    //   provider,
    //   sellToken
    // );
    const uniswapV2Router02Address = await contract.getContractAddress(types_1.ExternalContract.uniswapV2Router02, "homestead");
    const approve = await erc20instance.allowance(owner, uniswapV2Router02Address);
    if (Number(approve) > Number(amount)) {
        return true;
    }
    return false;
};
exports.hasApprovalBsc = async (owner, sellToken, amount, provider) => {
    const erc20instance = await contract.getErc20Address(provider, sellToken);
    // const exchangeAddress = await contract.getUniswapExchangeAddress(
    //   provider,
    //   sellToken
    // );
    // const uniswapV2Router02Address = await contract.getContractAddress(
    //   ExternalContract.uniswapV2Router02,
    //   "homestead"
    // );
    const approve = await erc20instance.allowance(owner, constants.PANCAKE_SWAP_ROUTER);
    if (Number(approve) > Number(amount)) {
        return true;
    }
    return false;
};
exports.isExchangeAvailable = async (token, provider) => {
    try {
        await contract.getUniswapExchangeAddress(provider, token);
        return true;
    }
    catch (e) {
        return false;
    }
};
exports.getUniswapLiquidity = async (tokenAddress, provider) => {
    try {
        const erc20instance = await contract.getErc20Address(provider, tokenAddress);
        const decimals = await erc20instance.decimals();
        const tokenToGet = new sdk_1.Token(sdk_1.ChainId.MAINNET, tokenAddress, decimals);
        const pair = await sdk_1.Fetcher.fetchPairData(tokenToGet, sdk_1.WETH[1]);
        const res0 = (await pair.reserveOf(tokenToGet));
        const res1 = (await pair.reserveOf(sdk_1.WETH[1]));
        const balances = {
            ETH: res1.toFixed(6),
            TOKEN: res0.toFixed(6),
        };
        return balances;
    }
    catch (e) {
        console.log("error get uniswap liquidity", tokenAddress, e);
        return 0;
    }
};
// -------------------- //
//     Transactions     //
// -------------------- //
exports.execTrade = async (sellToken, buyToken, sellAmount, buyAmount, wallet, txOptions) => {
    const exchange = exchanges.load(sellToken, buyToken);
    const tradeTx = await exchange.trade(sellAmount, buyAmount, wallet, txOptions);
    return tradeTx;
};
exports.execTradeBsc = async (sellToken, buyToken, sellAmount, buyAmount, wallet, txOptions) => {
    const exchange = exchanges.load(sellToken, buyToken, true);
    const tradeTx = await exchange.trade(sellAmount, buyAmount, wallet, txOptions);
    return tradeTx;
};
exports.execTradeFromSell = async (buyToken, sellAmount, buyAmount, destAddress, wallet, txOptions) => {
    const exchange = exchanges.load("ETH", buyToken);
    const tradeTx = await exchange.tradeFromSell(sellAmount, buyAmount, destAddress, wallet, txOptions);
    return tradeTx;
};
exports.execTrade_delayed = async (sellToken, buyToken, sellAmount, buyAmount, wallet, nonce, txOptions) => {
    const exchange = exchanges.load(sellToken, buyToken);
    const tradeTx = await exchange.trade_delayed(sellAmount, buyAmount, wallet, nonce, txOptions);
    return tradeTx;
};
// TO DO get token address call from staking smart contracts?
exports.sendCrypto = async (amount, toAddress, token, wallet, tokenAddress, txOptions) => {
    let tsx;
    const re = /B&/;
    const isBsc = re.test(token);
    if ((token === "ETH" && !isBsc) || (token === "B&BNB" && isBsc)) {
        txOptions.to = toAddress;
        txOptions.value = convert.ethToWeiBN(Number(amount));
        return wallet.sendTransaction(txOptions);
        // } else if ((token === "ETH" && !isBsc) || (token === "B&BNB" && isBsc )) {
    }
    else {
        const erc20instance = await contract.getErc20Address(wallet.provider, tokenAddress);
        let decimals;
        try {
            decimals = await erc20instance.decimals();
        }
        catch (e) {
            throw new Error("Unable to get decimals of the token");
        }
        const valueToSend = ethers_1.ethers.utils.parseUnits(amount, decimals);
        return erc20instance
            .connect(wallet)
            .transfer(toAddress, valueToSend, txOptions);
    }
};
exports.approveToken = async (token, wallet, txOptions) => {
    const uniswapV2Router02Address = await contract.getContractAddress(types_1.ExternalContract.uniswapV2Router02, "homestead");
    const erc20instance = await contract.getErc20Address(wallet.provider, token);
    return erc20instance.connect(wallet).approve(
    // exchangeAddress,
    uniswapV2Router02Address, ethers_1.ethers.utils.bigNumberify(2).pow(256).sub(1), txOptions);
};
exports.approveTokenBsc = async (token, wallet, txOptions) => {
    // const uniswapV2Router02Address = await contract.getContractAddress(
    //   ExternalContract.uniswapV2Router02,
    //   "homestead"
    // );
    const erc20instance = await contract.getErc20Address(wallet.provider, token);
    return erc20instance.connect(wallet).approve(constants.PANCAKE_SWAP_ROUTER, ethers_1.ethers.utils.bigNumberify(2).pow(256).sub(1), txOptions);
};
//# sourceMappingURL=wallet.js.map