"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ethers_1 = require("ethers");
const constants = __importStar(require("./constants"));
const providers = __importStar(require("./helpers/providers"));
const contract = __importStar(require("./helpers/contracts"));
const teller = __importStar(require("./core/teller"));
const shop = __importStar(require("./core/shop"));
const wallet = __importStar(require("./core/wallet"));
const util = __importStar(require("./core/util"));
const zone = __importStar(require("./core/zone"));
const constants_1 = require("./constants");
const types_1 = require("./types");
/**
 * This is the main detherJS object
 *
 */
class DetherJS {
    constructor(useMetamask) {
        this.usingMetamask = useMetamask;
        this.encryptedWallet = null;
        this.provider = null;
        this.bscProvider = null;
        this.network = null;
    }
    async init(connectOptions, bscConnectOptions) {
        // this.provider = this.usingMetamask
        //   ? await providers.connectMetamask()
        //   : await providers.connectEthers(connectOptions);
        this.provider = await providers.connectEthers(connectOptions);
        this.bscProvider = await providers.connectEthers(bscConnectOptions);
        this.network = await this.provider.getNetwork();
        this.protocolControllerContract = await contract.get(this.bscProvider, types_1.DetherContract.ProtocolController, constants_1.CONTRACT_ADDRESSES['bsc'][types_1.DetherContract.ProtocolController]);
        this.shopsContract = await contract.get(this.bscProvider, types_1.DetherContract.Shops, constants_1.CONTRACT_ADDRESSES['bsc'][types_1.DetherContract.Shops]);
        this.zoneFactoryContract = await contract.get(this.bscProvider, types_1.DetherContract.ZoneFactory, constants_1.CONTRACT_ADDRESSES['bsc'][types_1.DetherContract.ZoneFactory]);
        this.geoRegistryContract = await contract.get(this.bscProvider, types_1.DetherContract.GeoRegistry, constants_1.CONTRACT_ADDRESSES['bsc'][types_1.DetherContract.GeoRegistry]);
        this.dthContract = await contract.get(this.bscProvider, types_1.DetherContract.AnyswapV4ERC20, constants_1.CONTRACT_ADDRESSES['bsc'][types_1.DetherContract.AnyswapV4ERC20]);
    }
    async setCustomContractAddresses(contractAddresses) {
        constants.TICKER.custom.DTH = contractAddresses[types_1.DetherContract.DetherToken];
        constants.CONTRACT_ADDRESSES.custom.DetherToken =
            contractAddresses[types_1.DetherContract.DetherToken];
        constants.CONTRACT_ADDRESSES.custom.GeoRegistry =
            contractAddresses[types_1.DetherContract.GeoRegistry];
        constants.CONTRACT_ADDRESSES.custom.ZoneFactory =
            contractAddresses[types_1.DetherContract.ZoneFactory];
        constants.CONTRACT_ADDRESSES.custom.Zone =
            contractAddresses[types_1.DetherContract.Zone];
        constants.CONTRACT_ADDRESSES.custom.Shops =
            contractAddresses[types_1.DetherContract.Shops];
        // constants.CONTRACT_ADDRESSES.custom.TaxCollector =
        //   contractAddresses[DetherContract.TaxCollector];
    }
    async initProvider(connectOptions) {
        // this.provider = this.usingMetamask
        //   ? await providers.connectMetamask()
        //   : await providers.connectEthers(connectOptions);
        this.provider = await providers.connectEthers(connectOptions);
        this.network = await this.provider.getNetwork();
    }
    loadUser(encryptedWallet) {
        // no need for init() to first have been called
        // if (this.usingMetamask)
        //   throw new Error("cannot add encrypted wallet when using metamask");
        this.encryptedWallet = encryptedWallet;
    }
    async loadWallet(password, network) {
        // if (this.usingMetamask) {
        //   // source: https://docs.ethers.io/ethers.js/html/cookbook-providers.html?highlight=metamask
        //   //         https://docs.ethers.io/ethers.js/html/cookbook-providers.html?highlight=getsigner
        //   const signer = this.provider.getSigner();
        //   return signer;
        // }
        if (!this.encryptedWallet)
            throw new Error("did find no encrypted wallet");
        if (!password)
            throw new Error("need to pass in password as arg 1");
        console.log('load wallet Pre');
        const disconnectedWallet = await ethers_1.ethers.Wallet.fromEncryptedJson(this.encryptedWallet, password);
        const connectedWallet = new ethers_1.ethers.Wallet(disconnectedWallet.privateKey, network === 'BSC' ? this.bscProvider : this.provider);
        console.log('load wallet Post');
        return connectedWallet;
    }
    // -------------------- //
    //        Checks        //
    // -------------------- //
    hasProvider() {
        if (!this.provider)
            throw new Error("detherjs not yet initialized, first call init()");
    }
    hasWallet() {
        if (!this.encryptedWallet && !this.usingMetamask)
            throw new Error("no user loaded");
    }
    // -------------------- //
    //        Wallet        //
    // -------------------- //
    async getERC20Info(address, network) {
        this.hasProvider();
        return wallet.getERC20Info(address, network, this.provider, this.bscProvider);
    }
    async getAllBalance(address, tickers) {
        this.hasProvider();
        return wallet.getAllBalance(address, tickers, this.provider, this.bscProvider);
    }
    async getExchangeEstimation(sellToken, buyToken, sellAmount) {
        this.hasProvider();
        return wallet.getExchangeEstimation(sellToken, buyToken, sellAmount, this.provider);
    }
    async getExchangeEstimationBsc(sellToken, buyToken, sellAmount) {
        this.hasProvider();
        return wallet.getExchangeEstimationBsc(sellToken, buyToken, sellAmount, this.bscProvider);
    }
    async hasApproval(owner, sellToken, amount) {
        this.hasProvider();
        return wallet.hasApproval(owner, sellToken, amount, this.provider);
    }
    async hasApprovalBsc(owner, sellToken, amount) {
        this.hasProvider();
        return wallet.hasApprovalBsc(owner, sellToken, amount, this.bscProvider);
    }
    // sellAmount and buyAmount in string WEI
    async execExchange(password, sellToken, buyToken, sellAmount, buyAmount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password);
        return wallet.execTrade(sellToken, buyToken, sellAmount, buyAmount, userWallet, txOptions);
    }
    async execExchangeBsc(password, sellToken, buyToken, sellAmount, buyAmount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password, 'BSC');
        return wallet.execTradeBsc(sellToken, buyToken, sellAmount, buyAmount, userWallet, txOptions);
    }
    // sellAmount and buyAmount in string WEI
    // return a signed tx to broadcast later
    async execExchange_delayed(password, sellToken, buyToken, sellAmount, buyAmount, nonce, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password);
        return wallet.execTrade_delayed(sellToken, buyToken, sellAmount, buyAmount, userWallet, nonce, txOptions);
    }
    // Used from sell part . ONLY ETH -> ERC20
    async execExchangeFromSell(password, buyToken, sellAmount, buyAmount, destAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password);
        return wallet.execTradeFromSell(buyToken, sellAmount, buyAmount, destAddress, userWallet, txOptions);
    }
    async getAvailableToken(forLogo) {
        this.hasProvider();
        return wallet.getAvailableToken(this.provider, forLogo);
    }
    async getAvailableTokenDecimals(forLogo) {
        this.hasProvider();
        return wallet.getAvailableTokenDecimals(this.provider, forLogo);
    }
    async sendCrypto(password, toAddress, token, amount, tokenAddress = constants.ADDRESS_ZERO, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const re = /B&/;
        const isBsc = re.test(token);
        let userWallet;
        if (isBsc) {
            userWallet = await this.loadWallet(password, 'BSC');
        }
        else {
            userWallet = await this.loadWallet(password);
        }
        return wallet.sendCrypto(amount, toAddress, token, userWallet, tokenAddress, txOptions);
    }
    async signMessage(password, message) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password);
        return userWallet.signMessage(message);
    }
    async approveToken(password, token, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password);
        return wallet.approveToken(token, userWallet, txOptions);
    }
    async approveTokenBsc(password, token, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const userWallet = await this.loadWallet(password, 'BSC');
        return wallet.approveTokenBsc(token, userWallet, txOptions);
    }
    async getTokenLiquidity(tokenAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        return wallet.getUniswapLiquidity(tokenAddress, this.provider);
    }
    // -------------------- //
    //        Teller        //
    // -------------------- //
    async isTeller(address) {
        this.hasProvider();
        return teller.isTeller(address, this.bscProvider, this.zoneFactoryContract);
    }
    async getTeller(address) {
        this.hasProvider();
        return teller.getTeller(address, this.bscProvider, this.zoneFactoryContract);
    }
    async getTellerInZone(geohash6) {
        this.hasProvider();
        return teller.getTellerInZone(geohash6, this.bscProvider, this.zoneFactoryContract);
    }
    async getTellersInZones(geohash6List) {
        this.hasProvider();
        return teller.getTellersInZones(geohash6List, this.bscProvider, this.zoneFactoryContract);
    }
    async addTeller(password, tellerData, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return teller.addTeller(tellerData, wallet, this.zoneFactoryContract, txOptions);
    }
    async removeTeller(password, zoneGeohash, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return teller.removeTeller(zoneGeohash, wallet, this.zoneFactoryContract, txOptions);
    }
    async updateTeller(password, tellerData, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return teller.updateTeller(tellerData, wallet, this.zoneFactoryContract, txOptions);
    }
    async addTellerComment(password, zoneGeohash, ipfsHash, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return teller.addComment(zoneGeohash, ipfsHash, wallet, this.zoneFactoryContract, txOptions);
    }
    // -------------------- //
    //         Shop         //
    // -------------------- //
    async shopExistsByAddress(shopAddress) {
        this.hasProvider();
        return shop.existsByAddress(shopAddress, this.shopsContract);
    }
    async getShopByAddress(shopAddress) {
        this.hasProvider();
        return shop.getShopByAddress(shopAddress, this.shopsContract);
    }
    async getShopByPosition(geohash12) {
        this.hasProvider();
        return shop.getShopByPosition(geohash12, this.shopsContract);
    }
    async getShopsInZone(geohash6) {
        this.hasProvider();
        return shop.getShopsInZone(geohash6, this.shopsContract);
    }
    async getShopsInZones(geohash6List) {
        this.hasProvider();
        return shop.getShopsInZones(geohash6List, this.shopsContract);
    }
    async getLicencePriceInZone(geohash6) {
        this.hasProvider();
        return shop.getLicencePrice(geohash6, this.shopsContract);
    }
    async addShop(password, shopData, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.addShop(shopData, this.shopsContract, this.dthContract, wallet, txOptions);
    }
    // 1 eth address can only own 1 shop
    async removeShop(password, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.removeShop(this.shopsContract, wallet, txOptions);
    }
    async setShopLicencePrice(password, geohash6, newPrice, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.setShopLicencePrice(geohash6, newPrice, this.shopsContract, wallet, txOptions);
    }
    async topUpShop(password, topUpAmount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.topUpShop(topUpAmount, this.shopsContract, this.dthContract, wallet, txOptions);
    }
    async collectShopTaxes(password, geohash6, start, end, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.collectShopTaxes(geohash6, start, end, this.shopsContract, wallet, txOptions);
    }
    async deleteUserShop(password, geohash6, shopAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return shop.deleteUserShop(geohash6, shopAddress, this.shopsContract, wallet, txOptions);
    }
    // -------------------- //
    //         User         //
    // -------------------- //
    // -------------------- //
    //         Zone         //
    // -------------------- //
    async getCountryFloorPrice(country) {
        this.hasProvider();
        return zone.getCountryFloorPrice(country, this.protocolControllerContract, this.bscProvider);
    }
    async getGlobalParams() {
        this.hasProvider();
        return zone.getGlobalParams(this.protocolControllerContract, this.bscProvider);
    }
    async isZoneOwned(geohash6) {
        this.hasProvider();
        return zone.isZoneOwned(geohash6, this.zoneFactoryContract, this.bscProvider);
    }
    async getZoneByGeohash(geohash6) {
        this.hasProvider();
        return zone.getZoneByGeohash(geohash6, this.zoneFactoryContract, this.bscProvider);
    }
    async getZoneByAddress(address) {
        this.hasProvider();
        return zone.getZoneByAddress(address, this.bscProvider);
    }
    async getZonesStatus(geohash6List) {
        this.hasProvider();
        return zone.getZonesStatus(geohash6List, this.zoneFactoryContract, this.bscProvider);
    }
    async isBidderOnthisAuction(zoneAddress, ethAddress, auctionID) {
        this.hasProvider();
        return zone.isBidderOnthisAuction(zoneAddress, ethAddress, auctionID, this.bscProvider);
    }
    async createZone(password, country, geohash6, amount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.create(country, geohash6, amount, this.zoneFactoryContract, this.dthContract, wallet, txOptions);
    }
    async claimFreeZone(password, geohash6, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.claimFree(geohash6, wallet, this.zoneFactoryContract, this.dthContract, txOptions);
    }
    async bidZone(password, geohash6, bidAmount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.bid(geohash6, bidAmount, this.zoneFactoryContract, this.dthContract, wallet, txOptions);
    }
    async topUpZone(password, geohash6, topUpAmount, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.topUp(geohash6, topUpAmount, this.zoneFactoryContract, this.dthContract, wallet, txOptions);
    }
    async releaseZone(password, geohash6, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.release(geohash6, this.zoneFactoryContract, wallet, txOptions);
    }
    async withdrawZoneAuctionBid(password, geohash6, auctionId, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawFromAuction(geohash6, auctionId, this.zoneFactoryContract, wallet, txOptions);
    }
    async withdrawZoneAuctionsBid(password, geohash6, auctionIds, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawFromAuctions(geohash6, auctionIds, this.zoneFactoryContract, wallet, txOptions);
    }
    async withdrawZoneOwnerDth(password, geohash6, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password);
        return zone.withdrawDth(geohash6, this.zoneFactoryContract, wallet, txOptions);
    }
    async withdrawZoneAuctionBidAddress(password, zoneAddress, auctionId, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawFromAuctionAddress(zoneAddress, auctionId, wallet, txOptions);
    }
    async withdrawAuctionsRaw(password, zoneAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawAuctionsRaw(zoneAddress, wallet, txOptions);
    }
    async withdrawZoneAuctionsBidAddress(password, zoneAddress, auctionIds, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawFromAuctionsAddress(zoneAddress, auctionIds, wallet, txOptions);
    }
    async withdrawZoneOwnerDthAddress(password, zoneAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.withdrawDthAddress(zoneAddress, wallet, txOptions);
    }
    async processState(password, zoneAddress, txOptions = constants.DEFAULT_TX_OPTIONS) {
        this.hasProvider();
        this.hasWallet();
        const wallet = await this.loadWallet(password, 'BSC');
        return zone.processState(zoneAddress, wallet, txOptions);
    }
    async isZoneOpened(geohash6, country) {
        this.hasProvider();
        return zone.isZoneOpened(geohash6, country, this.geoRegistryContract);
    }
    async isZoneOwner(address) {
        this.hasProvider();
        return zone.isZoneOwner(address, this.zoneFactoryContract, this.bscProvider);
    }
    async getOpenBid(address) {
        this.hasProvider();
        return zone.getOpenBid(address, this.zoneFactoryContract);
    }
    // -------------------- //
    //        Util        //
    // -------------------- //
    async getTransactionStatus(txHash) {
        this.hasProvider();
        return util.getTransactionStatus(txHash, this.provider);
    }
    async getTransactionStatusBsc(txHash) {
        this.hasProvider();
        return util.getTransactionStatus(txHash, this.bscProvider);
    }
}
exports.default = DetherJS;
//# sourceMappingURL=dether.js.map