"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getErc20Address = exports.getErc20 = exports.get = exports.getUniswapExchangeAddress = exports.getContractAddress = exports.getAbi = void 0;
const ethers_1 = require("ethers");
const constants_1 = require("../constants");
const types_1 = require("../types");
const ABI = {
    // dether
    [types_1.DetherContract.DetherToken]: require("../../abi/dether/DetherToken.json")
        .abi,
    [types_1.DetherContract.ProtocolController]: require("../../abi/dether/ProtocolController.json")
        .abi,
    [types_1.DetherContract.AnyswapV4ERC20]: require("../../abi/dether/AnyswapV4ERC20.json")
        .abi,
    [types_1.DetherContract.GeoRegistry]: require("../../abi/dether/GeoRegistry.json")
        .abi,
    [types_1.DetherContract.Users]: require("../../abi/dether/Users.json").abi,
    [types_1.DetherContract.ZoneFactory]: require("../../abi/dether/ZoneFactory.json")
        .abi,
    [types_1.DetherContract.Shops]: require("../../abi/dether/Shops.json").abi,
    [types_1.DetherContract.Zone]: require("../../abi/dether/Zone.json").abi,
    [types_1.DetherContract.Teller]: require("../../abi/dether/Teller.json").abi,
    [types_1.DetherContract.CertifierRegistry]: require("../../abi/dether/CertifierRegistry.json")
        .abi,
    // [DetherContract.ShopDispute]: require("../../abi/dether/ShopDispute.json")
    // .abi,
    [types_1.DetherContract.TokenRegistry]: require("../../abi/dether/TokenRegistry.json")
        .abi,
    // [DetherContract.TaxCollector]: require("../../abi/dether/TaxCollector.json")
    //   .abi,
    // external
    [types_1.ExternalContract.erc20]: require("../../abi/external/erc20.json"),
    [types_1.ExternalContract.kyberNetworkProxy]: require("../../abi/external/kyberNetworkProxy.json"),
    [types_1.ExternalContract.uniswapExchange]: require("../../abi/external/uniswapExchange.json"),
    [types_1.ExternalContract.uniswapFactory]: require("../../abi/external/uniswapFactory.json"),
    [types_1.ExternalContract.uniswapV2Router02]: require("../../abi/external/uniswapV2Router02.json"),
    [types_1.ExternalContract.pancakeFactoryV2]: require("../../abi/external/pancakeFactoryV2.json"),
    [types_1.ExternalContract.pancakePair]: require("../../abi/external/pancakePair.json"),
    [types_1.ExternalContract.pancakeRouter]: require("../../abi/external/pancakeRouter.json"),
};
exports.getAbi = async (contractName) => {
    return ABI[contractName];
};
exports.getContractAddress = async (contractName, networkName) => {
    return constants_1.CONTRACT_ADDRESSES[networkName][contractName];
};
exports.getUniswapExchangeAddress = async (provider, token) => {
    const contractName = types_1.ExternalContract.uniswapFactory;
    let { name: networkName } = await provider.getNetwork();
    const uniswapFactoryInstance = await exports.get(provider, contractName);
    return uniswapFactoryInstance.getExchange(token);
};
exports.get = async (provider, contractName, address, overwriteAbi) => {
    if (!provider)
        throw new Error("missing provider arg");
    if (!contractName)
        throw new Error("missing contract name");
    const abi = ABI[contractName];
    if (!abi)
        throw new Error(`contract '${contractName}' is unknown`);
    if (!address) {
        // if no address passed in, load address from constants.js file
        let { name: networkName } = await provider.getNetwork();
        if (networkName === "unknown")
            networkName = "custom";
        address = constants_1.CONTRACT_ADDRESSES[networkName][contractName]; // tslint:disable-line
        if (!address)
            throw new Error(`could not find contract address of ${contractName} on ${networkName}`);
    }
    return new ethers_1.ethers.Contract(address, overwriteAbi || abi, provider);
};
exports.getErc20 = async (provider, tokenName) => {
    if (!provider)
        throw new Error("missing provider arg");
    if (!tokenName)
        throw new Error("missing tokenName");
    let { name: networkName } = await provider.getNetwork();
    if (networkName === "unknown")
        networkName = "custom";
    const address = constants_1.TICKER[networkName][tokenName]; // tslint:disable-line
    if (!address)
        throw new Error(`could not find token address of ${tokenName} on ${networkName}`);
    return new ethers_1.ethers.Contract(address, ABI.erc20, provider);
};
exports.getErc20Address = async (provider, address) => {
    if (!provider)
        throw new Error("missing provider arg");
    if (!address)
        throw new Error("missing address");
    let { name: networkName } = await provider.getNetwork();
    if (networkName === "unknown")
        networkName = "custom";
    return new ethers_1.ethers.Contract(address, ABI.erc20, provider);
};
//# sourceMappingURL=contracts.js.map