"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMaxUint256Value = exports.ipfsHashToBytes32 = exports.bytes32ToIpfsHash = exports.toNBytes = exports.remove0x = exports.add0x = exports.timestampNow = exports.toUtf8 = exports.stringToBytes = void 0;
const ethers_1 = require("ethers");
const utf8_1 = __importDefault(require("utf8"));
const bs58_1 = __importDefault(require("bs58"));
exports.stringToBytes = (str, len) => (ethers_1.ethers.utils.formatBytes32String(str).slice(0, 2 + (len * 2)) // 1 byte = 2 hex chars
);
exports.toUtf8 = (hex) => {
    if (hex.startsWith('0x'))
        hex = hex.slice(2); // tslint:disable-line
    let str = '';
    for (let i = 0; i < hex.length; i += 2) {
        const code = parseInt(hex.substr(i, 2), 16);
        if (code === 0)
            break;
        str += String.fromCharCode(code);
    }
    try {
        return utf8_1.default.decode(str);
    }
    catch (e) {
        return '';
    }
};
exports.timestampNow = () => (Math.floor(Date.now() / 1000));
exports.add0x = (txHash) => (txHash.startsWith('0x') ? txHash : `0x${txHash}`);
exports.remove0x = (val) => (val.replace('0x', ''));
exports.toNBytes = (str, n) => {
    let buffer = '';
    for (let i = 0; i < n; i += 1) {
        buffer += str[i] ? str[i].charCodeAt(0).toString(16) : '00';
    }
    return buffer;
};
// Add our default ipfs values for first 2 bytes:
// function:0x12=sha2, size:0x20=256 bits + cut off leading '0x'
exports.bytes32ToIpfsHash = (bytes32Hex) => (bs58_1.default.encode(Buffer.from(`1220${bytes32Hex.slice(2)}`, 'hex')));
// storing IPFS has as bytes32 helper functions --> https://ethereum.stackexchange.com/a/39961
exports.ipfsHashToBytes32 = (ipfsListing) => (`0x${bs58_1.default.decode(ipfsListing).slice(2).toString('hex')}`);
/**
 * return the max uint256 value, which is 2^256 - 1
 *
 * @return {BigNumber} max uint256 value as a BigNumber
 */
exports.getMaxUint256Value = () => (ethers_1.ethers.utils.bigNumberify(2).pow(256).sub(1));
//# sourceMappingURL=util.js.map